@@ -4,6 +4,9 @@
 from .updates import Update
 from .commands import BotCommandList
 from .chatmembers import ChatMember
+from .chatpermissions import ChatPermissions
+from .dice import Dice
+from .inputmedia import InputMediaList
 
 import json
 import requests
@@ -31,9 +34,12 @@ class TelegramBot:
 
         response = requests.get(self.__base_url + "/getMe").json()
         if not response['ok']:
+            # TODO get own id so bot cannot ban or restrict itself
+
             raise ValueError(f"Passed token not accepted by Telegram. Please check and try again: {response['description']}")
 
-    def poll(self):
+    def poll(self, one_time = False):
+
         while True:
             try:
                 update = requests.get(self.__base_url + "/getUpdates", data={'offset': self.__offset}).json()
@@ -61,72 +67,111 @@ class TelegramBot:
                     else:
                         return result, update
                 
+                if one_time:
+                    return result, update
+                
                 elif not self.__return_on_update_only:
                     return result, update
 
             except Exception as e:
                 print(f"An error occurred: {repr(e)}")
+    
+    def __setOffset(self, offset: int):
+        self.__offset = offset
 
-    def sendMessage(self, chat_id, message, keyboard: dict = None):
+    # USE WITH CAUTION!
+    def increaseOffset(self):
+        self.__offset += 1
+
+    def sendMessage(self, chat_id, message, keyboard: dict = None, markdown_style = False, silent = False):
 
         data = {
             'chat_id': chat_id,
             'text': message,
-            'reply_markup': json.dumps(keyboard) if keyboard else ""
+            'disable_notification': bool(silent),
+            'reply_markup': json.dumps(keyboard) if keyboard else "",
+            'parse_mode': "MarkdownV2" if markdown_style else ""
         }
 
         return requests.post(self.__base_url + "/sendMessage", data=data)
 
-    def sendPhoto(self, chat_id, photo_path, caption=""):
+    def sendPhoto(self, chat_id, photo_path, caption="", markdown_style = False, silent = False):
 
         data = {
             'chat_id': chat_id,
-            'caption': caption
+            'caption': caption,
+            'disable_notification': bool(silent),
+            'parse_mode': "MarkdownV2" if markdown_style else None
         }
 
         photo = {'photo': open(photo_path, 'rb')}
 
         return requests.post(self.__base_url + "/sendPhoto", data=data, files=photo)
 
-    def sendVideo(self, chat_id, video_path, caption="", duration: int = 0, width: int = 0, height: int = 0, thumbnail_path=""):
+    def sendVideo(self, chat_id, video_path, caption="", duration: int = 0, width: int = 0, height: int = 0, thumbnail_path="", markdown_style = False, silent = False):
         data = {
             'chat_id': chat_id,
             'caption': caption,
             'duration': duration if duration else None,
             'width': width if width else None,
-            'height': height if height else None
+            'height': height if height else None,
+            'disable_notification': bool(silent),
+            'parse_mode': "MarkdownV2" if markdown_style else ""
         }
 
-        media = {'photo': open(video_path, 'rb')}
+        media = {'video': open(video_path, 'rb')}
 
         if thumbnail_path:
             media['thumb'] = open(thumbnail_path, 'rb')
 
         return requests.post(self.__base_url + "/sendVideo", data=data, files=media)
 
-    def sendAudio(self, chat_id, audio_path, caption="", length: int = 0, performer: str = "", title: str = "", thumbnail_path=""):
+    def sendAnimation(self, chat_id, animation_path, caption="", duration: int = 0, width: int = 0, height: int = 0, thumbnail_path="", markdown_style = False, silent = False):
+        data = {
+            'chat_id': chat_id,
+            'caption': caption,
+            'duration': duration if duration else None,
+            'width': width if width else None,
+            'height': height if height else None,
+            'disable_notification': bool(silent),
+            'parse_mode': "MarkdownV2" if markdown_style else ""
+        }
+
+        media = {'animation': open(animation_path, 'rb')}
+
+        if thumbnail_path:
+            media['thumb'] = open(thumbnail_path, 'rb')
+
+        return requests.post(self.__base_url + "/sendAnimation", data=data, files=media)
+
+
+    def sendAudio(self, chat_id, audio_path, caption="", length: int = 0, performer: str = "", title: str = "", thumbnail_path="", markdown_style = False, silent = False):
         data = {
             'chat_id': chat_id,
             'caption': caption,
             'length': length if length else None,
             'performer': performer,
             'title': title,
+            'disable_notification': bool(silent),
+            'parse_mode': "MarkdownV2" if markdown_style else ""
         }
 
-        media = {'photo': open(audio_path, 'rb')}
+        media = {'audio': open(audio_path, 'rb')}
 
         if thumbnail_path:
             media['thumb'] = open(thumbnail_path, 'rb')
 
         return requests.post(self.__base_url + "/sendAudio", data=data, files=media)
 
-    def sendDocument(self, chat_id, document_path, caption="", thumbnail_path=""):
+    def sendDocument(self, chat_id, document_path, caption="", thumbnail_path="", markdown_style = False, silent = False):
         data = {
             'chat_id': chat_id,
-            'caption': caption
+            'caption': caption,
+            'disable_notification': bool(silent),
+            'parse_mode': "MarkdownV2" if markdown_style else ""
         }
 
-        media = {'photo': open(document_path, 'rb')}
+        media = {'document': open(document_path, 'rb')}
 
         if thumbnail_path:
             media['thumb'] = open(thumbnail_path, 'rb')
@@ -153,15 +198,75 @@ class TelegramBot:
             data['open_period'] = open_period
 
         return requests.post(self.__base_url + "/sendPoll", data=data)
+    
+    def sendDice(self, chat_id, emoji = Dice.DICE, silent = False):
+        
+        data = {
+            'chat_id': chat_id,
+            'emoji': emoji,
+            'disable_notification': bool(silent),
+        }
 
-    def __setOffset(self, offset: int):
-        self.__offset = offset
+        return requests.post(self.__base_url + "/sendDice", data=data)
 
-    # USE WITH CAUTION!
-    def increaseOffset(self):
-        self.__offset += 1
+    def sendContact(self, chat_id, phone_number, first_name, last_name = "", vcard = "", silent = False):
 
-    def editMessage(self, chat_id, message_id, text, keyboard: dict = None):
+        data = {
+            'chat_id': chat_id,
+            'phone_number': phone_number,
+            'first_name': first_name,
+            'last_name': last_name,
+            'vcard': vcard,
+            'disable_notification': bool(silent),
+        }
+
+        return requests.post(self.__base_url + "/sendContact", data=data)
+    
+    def sendLocation(self, chat_id, latitude, longitude, silent = False):
+
+        data = {
+            'chat_id': chat_id,
+            'latitude': latitude,
+            'longitude': longitude,
+            'disable_notification': bool(silent),
+        }
+
+        return requests.post(self.__base_url + "/sendLocation", data=data)
+
+    def sendVenue(self, chat_id, latitude, longitude, title, address = 0, silent = False):
+
+        data = {
+            'chat_id': chat_id,
+            'latitude': latitude,
+            'longitude': longitude,
+            'title': title,
+            'address': address,
+            'disable_notification': bool(silent),
+        }
+
+        return requests.post(self.__base_url + "/sendVenue", data=data)
+    
+    def sendChatAction(self, chat_id, chat_action, silent = False):
+        data = {
+            'chat_id': chat_id,
+            'action': chat_action,
+            'disable_notification': bool(silent)
+        }
+
+        return requests.post(self.__base_url + "/sendChatAction", data=data)
+
+    def sendMediaGroup(self, chat_id, media: InputMediaList, silent = False):
+        media_data, media_media = media.toDict()
+        
+        data = {
+            'chat_id': chat_id,
+            'media': json.dumps(media_data),
+            'disable_notification': bool(silent)
+        }
+        return requests.post(self.__base_url + "/sendMediaGroup", data=data, files=media_media)
+
+
+    def editMessage(self, chat_id, message_id, text, keyboard: dict = None, markdown_style = False):
 
         # you can only edit messages which do not have a reply-keyboard
 
@@ -169,7 +274,8 @@ class TelegramBot:
             'chat_id': chat_id,
             'message_id': message_id,
             'text': text,
-            'reply_markup': json.dumps(keyboard) if keyboard else ""
+            'reply_markup': json.dumps(keyboard) if keyboard else "",
+            'parse_mode': "MarkdownV2" if markdown_style else ""
         }
     
         return requests.post(self.__base_url + "/editMessageText", data=data)
@@ -236,7 +342,10 @@ class TelegramBot:
         return requests.post(self.__base_url + "/unbanChatMember", data=data)
     
     def kickChatMember (self, chat_id, user_id):
-        return self.unbanChatMember(chat_id, user_id, False)
+        response_ban = self.banChatMember(chat_id, user_id)
+        response_unban = self.unbanChatMember(chat_id, user_id, True)
+
+        return response_ban, response_unban
     
     def getChatMember (self, chat_id, user_id):
         data = {
@@ -267,4 +376,17 @@ class TelegramBot:
                 return [ChatMember(member) for member in response['result']]
         except Exception:
             return None
+    
+    def restrictChatMember (self, chat_id, user_id, permissions: ChatPermissions):
+        if not isinstance(permissions, ChatPermissions):
+            raise ValueError(f"passed parameter 'permissions' must be type {type(ChatPermissions)}, not {type(permissions)}")
+        
+        data = {
+            'chat_id': chat_id,
+            'user_id': user_id,
+            'permissions': json.dumps(permissions.toDict())
+        }
+
+        return requests.post(self.__base_url + "/restrictChatMember", data=data)
+
 
